use serde::Deserialize;

/// Declarative install configuration
#[derive(Debug, Deserialize)]
pub struct InstallConfig {
    /// Drive Configuration
    pub drive: DriveConfig,
    /// General Configuration
    pub general: GeneralConfig,
}

#[derive(Debug, Deserialize)]
pub struct DriveConfig {
    /// Boot Drive Path
    pub boot: String,
    /// Root Drive Path
    pub root: String,
}

#[derive(Debug, Deserialize)]
pub struct GeneralConfig {
    /// Enable encryption on root
    pub encryption: bool,
    /// Presets
    pub mode: InstallMode,
    /// System locale
    pub locale: String,
    /// Packages to install
    pub pkg: Vec<String>,
    /// Enable libvirt
    pub virtualization: bool,
    /// Enable docker
    pub docker: bool,
}

#[derive(Debug, Deserialize)]
pub enum InstallMode {
    /// Basic Arch Linux Installation
    Base,
    /// navOS Desktop
    Desktop,
    /// navOS Server
    Server,

    // TODO : Evaluate
    Kiosk,
}
