// TODO : Setup users
// TODO : Setup ssh (config + authorized_keys)
// TODO : Setup virtualization
// TODO : Setup docker

// DRIVE SELECTION

use crate::{
    config::{DriveConfig, GeneralConfig, InstallConfig},
    pkg, run_command,
};

pub fn str_vec(v: Vec<&str>) -> Vec<String> {
    v.into_iter().map(|x| x.to_string()).collect()
}

pub fn format_drives(conf: &DriveConfig) {
    // EFI (BOOT)
    run_command(
        &str_vec(vec!["mkfs.vfat", "-F", "32", conf.boot.as_str()]),
        None,
        false,
    );

    // ROOT
    run_command(
        &str_vec(vec!["cryptsetup", "luksFormat", conf.root.as_str()]),
        None,
        true,
    );
}

// MOUNT

pub fn mount_drives(conf: &DriveConfig) {
    run_command(
        &str_vec(vec!["cryptsetup", "open", conf.root.as_str(), "root"]),
        None,
        true,
    );

    run_command(
        &str_vec(vec!["mount", "/dev/mapper/root", "/mnt"]),
        None,
        false,
    );

    // TODO : Secure mount options
    run_command(
        &str_vec(vec!["mount", "--mkdir", conf.boot.as_str(), "/mnt/boot"]),
        None,
        false,
    );
}

// PACSTRAP

pub fn pacstrap(conf: &GeneralConfig) {
    // TODO : Modes install + pkgs

    let mut cmd: Vec<String> = vec!["pacstrap".into(), "-K".into(), "/mnt".into(), "base".into()];

    cmd.extend(conf.pkg.clone());

    run_command(&cmd, None, true);
}

// GENFSTAB

pub fn genfstab() {
    let (stdout, _) = run_command(&str_vec(vec!["genfstab", "-U", "/mnt"]), None, false);
    std::fs::write("/mnt/etc/fstab", stdout).unwrap();
}

pub fn first_boot_values(conf: &GeneralConfig) {
    // CHROOT

    // SYSTEMD-FIRSTBOOT

    // LOCALE
    // TODO : Logic for uncommenting a value
    std::fs::write("/etc/locale.gen", &conf.locale).unwrap();
    run_command(&str_vec(vec!["locale-gen"]), None, false);
}

pub fn setup_zram() {
    // arch-chroot /mnt pacman -S zram-generator
    std::fs::write(
        "/mnt/etc/systemd/zram-generator.conf",
        include_str!("root/zram-generator.conf"),
    )
    .unwrap();
    // arch-chroot /mnt systemctl enable --now systemd-zram-setup@zram0.service
}

// MKINITCPIO + UKI

pub fn setup_mkinitcpio() {
    std::fs::write(
        "/mnt/etc/mkinitcpio.d/linux.preset",
        include_str!("root/mkinitcpio/linux.preset"),
    )
    .unwrap();
    run_command(&str_vec(vec!["mkinitcpio", "--allpresets"]), None, true);
}

// SECURE BOOT

pub fn setup_secure_boot() {
    // TODO : Assert sb setup mode

    run_command(&vec!["sbctl".into(), "create-keys".into()], None, false);

    // TODO : Sign + Enroll
}

// MODS

/// Post Installer

// TPM Unlock

pub fn install_pkgs(pkg: &[&str]) {
    // TODO : implement
}

pub fn install(conf: InstallConfig) {
    // Drive Setup
    format_drives(&conf.drive);
    mount_drives(&conf.drive);

    // Base Install
    pacstrap(&conf.general);
    genfstab();

    // System Setup
    first_boot_values(&conf.general);
    // TODO : install bootloader

    match conf.general.mode {
        crate::config::InstallMode::Base => {}
        crate::config::InstallMode::Desktop => {
            install_pkgs(&pkg::DESKTOP_PKG);
        }
        crate::config::InstallMode::Server => {
            install_pkgs(&pkg::SERVER_PKG);
        }
        crate::config::InstallMode::Kiosk => {
            // TODO
        }
    }

    if conf.general.virtualization {
        // TODO : Enable virtualization
    }

    if conf.general.docker {
        // TODO : Enable docker
    }

    setup_zram();
    setup_secure_boot();
    setup_mkinitcpio();
}
